package java.io;

import java.util.Iterator;
import java.util.Vector;
import java.util.Collections;
import java.util.Enumeration;
import java.util.ArrayList;
import java.security.Permission;
import java.util.List;
import java.security.PermissionCollection;

final class FilePermissionCollection extends PermissionCollection implements Serializable
{
    private transient List<Permission> perms;
    private static final long serialVersionUID = 2202956749081564585L;
    private static final ObjectStreamField[] serialPersistentFields;
    
    public FilePermissionCollection() {
        this.perms = new ArrayList();
    }
    
    @Override
    public void add(Permission obj) {
        if (!(obj instanceof FilePermission)) {
            throw new IllegalArgumentException("invalid permission: " + obj);
        }
        if (this.isReadOnly()) {
            throw new SecurityException("attempt to add a Permission to a readonly PermissionCollection");
        }
        synchronized (this) {
            this.perms.add(obj);
        }
        return;
    }
    
    @Override
    public boolean implies(Permission permission) {
        if (!(permission instanceof FilePermission)) {
            return false;
        }
        FilePermission filePermission = (FilePermission)permission;
        int mask = filePermission.getMask();
        int n = 0;
        synchronized (this) {
            for (int size = this.perms.size(), i = 0; i < size; ++i) {
                FilePermission filePermission2 = (FilePermission)this.perms.get(i);
                if ((mask & filePermission2.getMask()) != 0x0 && filePermission2.impliesIgnoreMask(filePermission)) {
                    n |= filePermission2.getMask();
                    if ((n & mask) == mask) {
                        return true;
                    }
                    mask ^= n;
                }
            }
        }
        return false;
    }
    
    @Override
    public Enumeration<Permission> elements() {
        synchronized (this) {
            return Collections.enumeration(this.perms);
        }
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        Vector vector = new Vector(this.perms.size());
        synchronized (this) {
            vector.addAll(this.perms);
        }
        objectOutputStream.putFields().put("permissions", vector);
        objectOutputStream.writeFields();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        Vector<Object> vector = (Vector)objectInputStream.readFields().get("permissions", null);
        this.perms = new ArrayList(vector.size());
        Iterator<Object> iterator = vector.iterator();
        while (iterator.hasNext()) {
            this.perms.add((Permission)iterator.next());
        }
    }
    
    static {
        serialPersistentFields = new ObjectStreamField[] { new ObjectStreamField("permissions", Vector.class) };
    }
}
